import { __decorate } from 'tslib';
import { Injectable, ɵɵdefineInjectable, ɵɵinject, ElementRef, HostBinding, Input, Component, ContentChildren, ChangeDetectorRef, ViewContainerRef, Renderer2, ViewChild, EventEmitter, Output, Pipe, HostListener, Directive, NgModule } from '@angular/core';
import { FormBuilder, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import { DatePipe, CommonModule } from '@angular/common';
import { isNullOrUndefined, isObject, isArray } from 'util';
import { Observable } from 'rxjs';
import { debounceTime, map } from 'rxjs/operators';

const TRANSLATION = {};

const TRANSLATION$1 = {
    TITULO: {
        ptBR: 'TITULO'
    }
};

var AgTableFilterMode;
(function (AgTableFilterMode) {
    AgTableFilterMode["CONTAINS"] = "contains";
    AgTableFilterMode["EQUALS"] = "equals";
    AgTableFilterMode["STARTS_WITH"] = "startsWith";
    AgTableFilterMode["ENDS_WITH"] = "endsWith";
})(AgTableFilterMode || (AgTableFilterMode = {}));

const TRANSLATION$2 = {
    ALL_OPTION: {
        ptBR: 'Todos',
        enUS: 'All'
    },
    DATE_FORMAT: {
        ptBR: 'dd/MM/yyyy',
        enUS: 'yyyy/MM/dd'
    }
};

var AgTableFilterType;
(function (AgTableFilterType) {
    AgTableFilterType["NONE"] = "none";
    AgTableFilterType["TEXT"] = "text";
    AgTableFilterType["SELECT"] = "select";
    AgTableFilterType["DATE"] = "date";
    AgTableFilterType["CURRENCY"] = "currency";
    // NUMBER = 'number',
})(AgTableFilterType || (AgTableFilterType = {}));

let Helper = class Helper {
    constructor() {
        this.datePipe = new DatePipe('en-US');
    }
    padLeft(text, qtty = 2, char = 0) {
        if (!isNullOrUndefined(text))
            while (text.length < qtty)
                text = char + text;
        return text;
    }
    padRight(text, qtty = 2, char = 0) {
        if (!isNullOrUndefined(text))
            while (text.length < qtty)
                text += char;
        return text;
    }
    randomNumber(min = 1, max = 9999) {
        let number = Math.floor(Math.random() * max);
        return Math.max(number, min);
    }
    onlyNumbers(text, exceptions = null) {
        if (!isNullOrUndefined(text)) {
            let expression = (exceptions && exceptions.length) ? ('[^\\d|' + exceptions.join('|') + ']') : '[^\\d]';
            return text.replace(new RegExp(expression, 'g'), '');
        }
        return '';
    }
    onlyAlphaNumeric(text) {
        if (!isNullOrUndefined(text))
            return text.replace(/[^a-zA-Z0-9]+/g, '');
        return '';
    }
    onlyNumberAndToFloat(text) {
        if (text) {
            let strNumber = text.replace(/[^\d.\d]/g, '');
            let number = parseFloat(strNumber);
            if (!isNaN(number))
                return number;
        }
        return 0;
    }
    removeAccents(text) {
        return text ? text.normalize('NFD').replace(/[\u0300-\u036f]/g, '') : '';
    }
    dateIsValid(date, dateFormat) {
        return !isNullOrUndefined(this.strToDate(date, dateFormat));
    }
    toAmericanDate(date, separator = '-') {
        if (date instanceof Date)
            date = this.dateToStr(date);
        if (this.dateIsValid(date)) {
            let dateStr = date;
            if (dateStr && (dateStr.includes('-') || dateStr.includes('/'))) {
                if (dateStr.includes('/'))
                    dateStr = dateStr.replace(/\//g, '-');
                const dateSplited = dateStr.split('-');
                if (dateSplited.length >= 3) {
                    const year = parseInt((dateSplited[0].length === 4) ? dateSplited[0] : dateSplited[2]);
                    const month = parseInt(dateSplited[1]);
                    const day = parseInt((dateSplited[0].length === 4) ? dateSplited[2] : dateSplited[0]);
                    const arrayDate = [year, (month > 9 ? month : ('0' + month)), (day > 9 ? day : ('0' + day))];
                    return arrayDate.join(separator);
                }
            }
        }
        return date;
    }
    strToDate(date, dateFormat) {
        if (typeof date === 'string' && (date.length >= 8 || dateFormat)) {
            let dateStr = date;
            if (dateStr && (dateStr.includes('-') || dateStr.includes('/'))) {
                if (dateStr.includes('/'))
                    dateStr = dateStr.replace(/\//g, '-');
                dateStr = dateStr.substr(0, 10);
                if (dateFormat)
                    dateFormat = dateFormat.replace(/\//g, '-');
                const dateSplited = dateStr.split('-');
                const length = dateSplited.length;
                if (length >= 1) {
                    let year = 1111;
                    let month = 11;
                    let day = 11;
                    if (length >= 3) {
                        year = parseInt((dateSplited[0].length === 4) ? dateSplited[0] : dateSplited[2]);
                        month = parseInt(dateSplited[1]);
                        day = parseInt((dateSplited[0].length === 4) ? dateSplited[2] : dateSplited[0]);
                    }
                    else if (dateFormat) {
                        let i = 0;
                        for (let format of dateFormat.split('-')) {
                            try {
                                if (format.toLowerCase().startsWith('y'))
                                    year = parseInt(dateSplited[i]);
                                else if (format.toLowerCase().startsWith('m'))
                                    month = parseInt(dateSplited[i]);
                                else if (format.toLowerCase().startsWith('d'))
                                    day = parseInt(dateSplited[i]);
                            }
                            catch (_a) { }
                            i++;
                        }
                    }
                    const arrayDate = [year, (month > 9 ? month : ('0' + month)), (day > 9 ? day : ('0' + day))];
                    dateStr = arrayDate.join('-');
                    if (dateStr)
                        return new Date(year, month - 1, day, 0, 0, 0);
                }
            }
        }
        else if (date instanceof Date)
            return date;
        return null;
    }
    setDaysToDate(date, days) {
        if (typeof date === 'string')
            date = this.strToDate(date);
        else if (date)
            date = this.copyDate(date);
        if (date && typeof days === 'number')
            return new Date(date.setDate(date.getDate() + days));
        return date;
    }
    dateToStr(date, format = 'dd/MM/yyyy') {
        if (!format)
            format = 'dd/MM/yyyy';
        format = format.replace(/mm/g, 'MM');
        if (date) {
            if (typeof date === 'string')
                date = this.strToDate(date);
            return this.datePipe.transform(date, format);
        }
        return null;
    }
    getDateToday(withHours = false) {
        let today = new Date();
        if (withHours)
            return today;
        else
            return new Date(today.getFullYear(), today.getMonth(), today.getDate(), 0, 0, 0);
    }
    getYearOfDate(date) {
        date = this.strToDate(date);
        if (date)
            return date.getFullYear();
        return 0;
    }
    getMonthOfDate(date) {
        date = this.strToDate(date);
        if (date)
            return date.getMonth() + 1;
        return 0;
    }
    getDayOfDate(date) {
        date = this.strToDate(date);
        if (date)
            return date.getDate();
        return 0;
    }
    copyDate(date) {
        if (date)
            return new Date(date.toString());
        return date;
    }
    dateFormat(date, format = 'dd/MM/yyyy') {
        date = this.strToDate(date);
        if (date)
            return this.datePipe.transform(date, format);
        return null;
    }
    strToDoubleCurrency(strCurrency) {
        strCurrency = strCurrency.replace(/[,.]/g, (m) => m === ',' ? '.' : ',');
        return Number(strCurrency);
    }
    copy(obj) {
        if (obj)
            return JSON.parse(JSON.stringify(obj));
        else
            return obj;
    }
    copyToClipboard(inputElement) {
        inputElement.focus();
        inputElement.select();
        let successful = document.execCommand('copy');
    }
    getDiffOfDate(dateBegin, dateEnd, returnType = 'days') {
        if (dateBegin && dateEnd) {
            let delta = Math.abs(dateEnd.getTime() - dateBegin.getTime()) / 1000;
            let days = Math.floor(delta / 86400);
            delta -= days * 86400;
            let hours = Math.floor(delta / 3600) % 24;
            delta -= hours * 3600;
            let minutes = Math.floor(delta / 60) % 60;
            delta -= minutes * 60;
            let seconds = Math.floor(delta % 60);
            if (returnType === 'days')
                return days;
            else if (returnType === 'hours')
                return hours;
            else if (returnType === 'minutes')
                return minutes;
            else if (returnType === 'seconds')
                return seconds;
        }
        return 0;
    }
    stringReplace(source, str, strToRep) {
        return source.replace(str, strToRep).toString();
    }
};
Helper = __decorate([
    Injectable()
], Helper);

class AgTableSettingsAbstract {
    constructor() {
        this.lang = 'en-US';
        this.filterDebounceTime = 500;
    }
}

let AgTableCustomSettings = class AgTableCustomSettings extends AgTableSettingsAbstract {
};
AgTableCustomSettings.ɵprov = ɵɵdefineInjectable({ factory: function AgTableCustomSettings_Factory() { return new AgTableCustomSettings(); }, token: AgTableCustomSettings, providedIn: "root" });
AgTableCustomSettings = __decorate([
    Injectable({
        providedIn: 'root'
    })
], AgTableCustomSettings);

let AgTableSettings = class AgTableSettings extends AgTableSettingsAbstract {
    constructor(customSettings) {
        super();
        if (customSettings)
            Object.assign(this, customSettings);
    }
};
AgTableSettings.ctorParameters = () => [
    { type: AgTableCustomSettings }
];
AgTableSettings.ɵprov = ɵɵdefineInjectable({ factory: function AgTableSettings_Factory() { return new AgTableSettings(ɵɵinject(AgTableCustomSettings)); }, token: AgTableSettings, providedIn: "root" });
AgTableSettings = __decorate([
    Injectable({
        providedIn: 'root'
    })
], AgTableSettings);

let AgTableLangService = class AgTableLangService {
    constructor(settings) {
        this.settings = settings;
    }
    set lang(value) { this.settings.lang = value; }
    get lang() { return this.settings.lang; }
    get langTranslation() { return this.lang.replace(/-/g, '').trim(); }
    setLang(lang, reloadPage = false) {
        this.lang = lang;
        if (reloadPage)
            location.reload();
    }
    getLang() { return this.lang; }
    getText(value, dict, asString = true) {
        let getText = (arg) => {
            let observ = arg ? arg : null;
            if (!dict || JSON.stringify(dict) !== '{}') {
                let helper = this.processValue(value, dict);
                // tslint:disable-next-line:quotemark
                if (typeof helper === "object") {
                    let text = `-- WITHOUT TRANSLATION [${value}] --`;
                    if (observ)
                        observ.next(text);
                    else
                        return text;
                }
                else {
                    if (observ)
                        observ.next(helper);
                    else
                        return helper;
                }
            }
            else {
                if (observ) {
                    if (dict && value in dict && this.langTranslation in dict[value])
                        observ.next(dict[value][this.langTranslation]);
                    else
                        observ.next('');
                }
                else {
                    if (dict && value in dict && this.langTranslation in dict[value])
                        return dict[value][this.langTranslation];
                    else
                        return '';
                }
            }
            if (observ)
                return () => observ.unsubscribe();
        };
        if (!asString)
            return new Observable(getText);
        else
            return getText();
    }
    processValue(prop, dict) {
        const positions = prop.split('.');
        let helper = null;
        if (dict && JSON.stringify(dict) !== '{}') {
            helper = dict;
            for (let _prop of positions) {
                if (_prop in helper)
                    helper = helper[_prop];
            }
            if (this.langTranslation in helper)
                helper = helper[this.langTranslation];
        }
        return helper ? helper : '';
    }
};
AgTableLangService.ctorParameters = () => [
    { type: AgTableSettings }
];
AgTableLangService.ɵprov = ɵɵdefineInjectable({ factory: function AgTableLangService_Factory() { return new AgTableLangService(ɵɵinject(AgTableSettings)); }, token: AgTableLangService, providedIn: "root" });
AgTableLangService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], AgTableLangService);

let AgTableColComponent = class AgTableColComponent {
    constructor(helper, langService, fb, elRef) {
        this.helper = helper;
        this.langService = langService;
        this.fb = fb;
        this.elRef = elRef;
        this._class = true;
        this.width = null;
        this.filter = AgTableFilterType.NONE;
        this.field = '';
        this.mode = null;
        this.placeholder = '';
        this.dateFormat = '';
        this.optionAllLabel = '';
        this.options = [];
        /* DISABLED
        @Input('ready-only') public readOnly: boolean = false;
        @Input('min-date') public minDate: Date;
        @Input('max-date') public maxDate: Date;
        @Input('date-separator') public dateSeparator: string = '/'; */
        this.noSort = false;
        this.noTruncate = false;
        this.sort = null;
        this.disableFilter = false;
        this.filterValue = null;
        this.dictionary = TRANSLATION$2;
        this.colIndex = 0;
        this.canFiltering = false;
        this.fields = [];
        this.filterActive = false;
    }
    set _width(value) {
        if (this.el) {
            this.el.style.width = value;
            if (value && value.endsWith('%'))
                this.el.style.maxWidth = ((this.helper.onlyNumberAndToFloat(value) * this.parent.el.clientWidth) / 100) + 'px';
            else
                this.el.style.maxWidth = value;
        }
    }
    get _width() { return (this.elRef && this.el) ? this.el.style.width : 'auto'; }
    get el() { return (this.elRef && this.elRef.nativeElement) ? this.elRef.nativeElement : null; }
    get canFilter() {
        return (this.filter && this.filter !== AgTableFilterType.NONE && this.filter.toString() !== '') ||
            (this.customFilter && this.customFilter.length > 0);
    }
    get isSorting() { return this.parent && this.parent.colSorting && this.parent.colSorting.col === this; }
    get isSortAsc() { return this.isSorting && this.parent.colSorting.asc; }
    ngOnInit() {
    }
    ngAfterViewInit() {
        if (!this.optionAllLabel)
            this.optionAllLabel = this.langService.getText('ALL_OPTION', this.dictionary);
        if (!this.dateFormat)
            this.dateFormat = this.langService.getText('DATE_FORMAT', this.dictionary);
        if (!this.noSort && !this.field && (!this.customFilter || !this.customFilter.length)) {
            this.filter = AgTableFilterType.NONE;
            console.warn(`The ordering will not applied because the [field] of column has not been defined.`);
        }
    }
    ngOnChanges(changes) {
        if (!changes || 'width' in changes) {
            if (!this.widthIsValid())
                this.width = 'auto';
            if (this.widthIsValid() && this.helper.onlyNumbers(this.width) === this.width)
                this.width += 'px';
            if (this.parent && this.parent.cols) {
                let unit = this.getWidthUnit();
                if (unit !== '' && this.parent.cols.some((x) => x.getWidthUnit() !== '' && x.getWidthUnit() !== unit)) {
                    let anotherUnit = this.parent.cols.find((x) => x.getWidthUnit() !== '' && x.getWidthUnit() !== unit).getWidthUnit();
                    console.warn(`A another column has width defined with like "${anotherUnit}". So this [width] was redefined as "auto".`);
                    this.width = 'auto';
                    this.setWidth();
                    return;
                }
            }
            if (this.getWidthUnit() === '%') {
                if (parseFloat(this.width.replace(this.getWidthUnit(), '')) >= 100) {
                    console.warn(`The columns cannot have width equal or greater than 100%.`);
                    this.width = 'auto';
                    this.setWidth();
                    return;
                }
            }
            this.setWidth();
        }
        if (changes) {
            if ('noSort' in changes)
                this.noSort = (typeof this.noSort === 'string' && (this.noSort === '' || this.noSort === 'true')) ? true : this.noSort;
            if ('noTruncate' in changes)
                this.noTruncate = (typeof this.noTruncate === 'string' && (this.noTruncate === '' || this.noTruncate === 'true')) ? true : this.noTruncate;
            if ('sort' in changes) {
                this.sort = !this.sort ? null : this.sort;
                if (this.sort !== 'asc' && this.sort !== 'desc')
                    this.sort = null;
                if (this.parent) {
                    this.parent.colSorting = {
                        col: this,
                        asc: this.sort === 'asc'
                    };
                    this.sort = null;
                    this.parent.parent.onSortChange();
                }
            }
            if ('filter' in changes) {
                if (!this.filter && this.filter.toString() !== '')
                    this.filter = AgTableFilterType.NONE;
                else if (this.filter.toString() === '')
                    this.filter = AgTableFilterType.TEXT;
                if (this.filter !== AgTableFilterType.NONE && !this.mode) {
                    switch (this.filter) {
                        case AgTableFilterType.TEXT:
                        case AgTableFilterType.CURRENCY:
                            this.mode = AgTableFilterMode.CONTAINS;
                            break;
                        case AgTableFilterType.SELECT:
                        case AgTableFilterType.DATE:
                            this.mode = AgTableFilterMode.EQUALS;
                            break;
                        default:
                            this.mode = AgTableFilterMode.CONTAINS;
                            break;
                    }
                }
                if (this.parent) {
                    if (this.field && this.parent.filterCtrls[this.field]) {
                        if (this.disableFilter)
                            this.parent.filterCtrls[this.field].disable();
                        else
                            this.parent.filterCtrls[this.field].enable();
                    }
                    else if (this.customFilter)
                        this.customFilter.forEach((custom) => {
                            if (this.disableFilter)
                                this.parent.filterCtrls[custom.field].disable();
                            else
                                this.parent.filterCtrls[custom.field].enable();
                        });
                }
                if (this.canFilter && !this.field) {
                    this.filter = AgTableFilterType.NONE;
                    console.warn(`The filter will not applied because the [field] of column has not been defined.`);
                }
                setTimeout(() => {
                    if (this.parent)
                        this.parent.parent.definePaddingTop();
                });
            }
            if ('customFilter' in changes) {
                if (this.canFilter && this.field) {
                    console.warn(`The [custom-filter] will not applied because the [filter] has already been defined.`);
                    this.customFilter = null;
                    return;
                }
                else {
                    if (this.customFilter && (isObject(this.customFilter) || typeof this.customFilter === 'string')) {
                        if (isObject(this.customFilter) && !isArray(this.customFilter))
                            this.customFilter = [this.customFilter];
                        else if (isArray(this.customFilter) && (this.customFilter.length > 0 && typeof this.customFilter[0] === 'string'))
                            this.customFilter = this.customFilter.map(field => ({ field }));
                        else if (!isObject(this.customFilter))
                            this.customFilter = [{ field: this.customFilter }];
                        if (!isArray(this.customFilter) || !this.customFilter.every(x => (isObject(x) && x.field) ? true : false)) {
                            console.warn(`The [custom-filter] will not applied because field has not been informed or is invalid. The value should look like this: { field: string, mode?: string }`);
                            this.customFilter = null;
                        }
                        else {
                            if (!this.noSort)
                                this.noSort = this.customFilter.length > 1;
                            this.customFilter.forEach((filter) => {
                                if (!filter.mode)
                                    filter.mode = AgTableFilterMode.CONTAINS;
                            });
                            setTimeout(() => {
                                if (this.parent)
                                    this.parent.parent.definePaddingTop();
                            });
                        }
                    }
                    else {
                        console.warn(`The [custom-filter] will not applied because value is invalid. The value should look like this: { field: string, mode?: string }`);
                        this.customFilter = null;
                    }
                }
            }
        }
    }
    makeFormControl(defaultValue = null) {
        return this.fb.control(defaultValue);
    }
    onFiltering() {
        if (this.parent && this.parent.parent) {
            if (this.parent) {
                if (this.field)
                    this.filterActive = (this.parent.filterCtrls[this.field] && this.parent.filterCtrls[this.field].value !== null && this.parent.filterCtrls[this.field].value !== '');
                else if (this.customFilter)
                    this.filterActive = this.customFilter.some((custom) => {
                        return this.parent.filterCtrls[custom.field] && this.parent.filterCtrls[custom.field].value !== null && this.parent.filterCtrls[custom.field].value !== '';
                    });
            }
            else
                this.filterActive = false;
            this.parent.parent.onFilterChange();
        }
    }
    onFilterRender() {
        if (this.parent)
            this.parent.parent.definePaddingTop();
    }
    toggleSort() {
        if (this.parent && this.parent.parent && !this.parent.parent.loading) {
            let asc;
            if (this.isSorting) {
                if (this.parent.colSorting.asc) {
                    asc = false;
                    this.parent.colSorting = {
                        col: this,
                        asc: asc
                    };
                }
                else {
                    this.parent.colSorting = null;
                }
            }
            else {
                asc = true;
                this.parent.colSorting = {
                    col: this,
                    asc: asc
                };
            }
            this.parent.parent.onSortChange();
        }
    }
    widthIsValid() {
        return !isNullOrUndefined(this.width) && (typeof this.width === 'string') && this.width !== '' && this.width !== 'auto';
    }
    getWidthUnit() {
        return this.widthIsValid() ? this.width.replace(this.helper.onlyNumbers(this.width), '') : '';
    }
    onRender(parent, index) {
        this.parent = parent;
        this.colIndex = index;
        this.canFiltering = this.canFilter && !isNullOrUndefined(this.parent.filterCtrls);
        this.fields = this.generateFieldsFilter();
        this.setWidth();
        this.ngOnChanges(null);
    }
    generateFieldsFilter() {
        if (this.canFilter) {
            if (this.field)
                return [this.field];
            else
                return this.customFilter.map((custom) => custom.field);
        }
        return [];
    }
    setWidth() {
        if (this.parent && this.parent.parent.body.itemsContainerEl && this.parent.parent.body.itemsContainerEl.nativeElement) {
            if (!this.widthIsValid()) {
                let unit = '';
                if (this.parent.cols.some((x, i) => x.getWidthUnit() !== ''))
                    unit = this.parent.cols.find((x, i) => x.getWidthUnit() !== '').getWidthUnit();
                if (unit === 'px') {
                    let colsWithWidth = this.parent.cols.filter((x, i) => i !== this.colIndex && x.widthIsValid());
                    let widthDiff = colsWithWidth.reduce((prev, curr) => prev + parseFloat(this.helper.onlyNumbers(curr.width)), 0);
                    let containerWidth = this.parent.el.clientWidth;
                    this._width = (containerWidth - widthDiff) / Math.abs(this.parent.cols.length - colsWithWidth.length) + 'px';
                }
                else {
                    if (!unit)
                        unit = '%';
                    let colsWithWidth = this.parent.cols.filter((x, i) => i !== this.colIndex && x.widthIsValid());
                    let widthDiff = colsWithWidth.reduce((prev, curr) => prev + parseFloat(this.helper.onlyNumbers(curr.width)), 0);
                    this._width = (100 - widthDiff) / Math.abs(this.parent.cols.length - colsWithWidth.length) + unit;
                }
            }
            else
                this._width = this.width;
        }
        else
            this._width = 'auto';
        this.setCellsWidth();
    }
    setCellsWidth() {
        if (this.parent && this.parent.parent.body && this.parent.parent.body.rows.length) {
            this.parent.parent.body.rows.forEach(row => {
                let cell = row.cells.find(cell => cell.cellIndex === this.colIndex);
                if (cell)
                    cell.setWidth();
            });
        }
    }
};
AgTableColComponent.ctorParameters = () => [
    { type: Helper },
    { type: AgTableLangService },
    { type: FormBuilder },
    { type: ElementRef }
];
__decorate([
    HostBinding('class.ag-table-col')
], AgTableColComponent.prototype, "_class", void 0);
__decorate([
    Input()
], AgTableColComponent.prototype, "width", void 0);
__decorate([
    Input('custom-filter')
], AgTableColComponent.prototype, "customFilter", void 0);
__decorate([
    Input()
], AgTableColComponent.prototype, "filter", void 0);
__decorate([
    Input()
], AgTableColComponent.prototype, "field", void 0);
__decorate([
    Input()
], AgTableColComponent.prototype, "mode", void 0);
__decorate([
    Input()
], AgTableColComponent.prototype, "placeholder", void 0);
__decorate([
    Input('date-format')
], AgTableColComponent.prototype, "dateFormat", void 0);
__decorate([
    Input('option-all-label')
], AgTableColComponent.prototype, "optionAllLabel", void 0);
__decorate([
    Input()
], AgTableColComponent.prototype, "options", void 0);
__decorate([
    Input('no-sort')
], AgTableColComponent.prototype, "noSort", void 0);
__decorate([
    Input('no-truncate')
], AgTableColComponent.prototype, "noTruncate", void 0);
__decorate([
    Input('sort')
], AgTableColComponent.prototype, "sort", void 0);
__decorate([
    Input('disable-filter')
], AgTableColComponent.prototype, "disableFilter", void 0);
__decorate([
    Input('filter-value')
], AgTableColComponent.prototype, "filterValue", void 0);
AgTableColComponent = __decorate([
    Component({
        selector: 'ag-table-col',
        template: "<span *ngIf=\"!noSort\" class=\"ag-table-col-sort\" (click)=\"toggleSort()\" [ngClass]=\"{ 'sort-none': !isSorting, 'sort-asc': isSorting && isSortAsc, 'sort-desc': isSorting && !isSortAsc }\"></span>\n\n<div class=\"w-auto\" [ngClass]=\"{ 'text-truncate': !noTruncate && !customFilter }\">\n    <ng-content></ng-content>\n</div>\n\n<div *ngIf=\"canFiltering && parent?.filterCtrls\">\n    <ag-table-filter *ngFor=\"let field of fields\"\n        [frmControl]=\"parent?.filterCtrls[field]\"\n        [type]=\"filter\"\n        [placeholder]=\"placeholder\"\n        [dateFormat]=\"dateFormat\"\n        [optionAllLabel]=\"optionAllLabel\"\n        [options]=\"options\"\n        (onChange)=\"onFiltering()\" \n        (onRender)=\"onFilterRender()\"> \n    </ag-table-filter>\n</div>"
    })
], AgTableColComponent);

let AgTableHeaderComponent = class AgTableHeaderComponent {
    constructor(elRef, helper, fb) {
        this.elRef = elRef;
        this.helper = helper;
        this.fb = fb;
        this.class = true;
        this.dictionary = TRANSLATION$1;
        this.colSorting = null;
        this.frmFilter = this.fb.group({ none: [null] });
    }
    get cols() { return this.queryCols ? this.queryCols.toArray() : []; }
    get height() { return (this.el) ? `${this.el.children[0].clientHeight}px` : '0px'; }
    get visible() { return (this.elRef && this.el) ? this.el.style.visibility !== 'hidden' : false; }
    set visible(value) {
        if ((this.elRef && this.el))
            this.el.style.visibility = (value ? '' : 'hidden');
    }
    get filterCtrls() { return (this.frmFilter && this.frmFilter.controls) ? this.frmFilter.controls : null; }
    get el() { return (this.elRef && this.elRef.nativeElement) ? this.elRef.nativeElement : null; }
    ngOnInit() {
    }
    ngAfterViewInit() {
    }
    makeForm() {
        let objControls = {};
        this.cols.forEach((col) => {
            if (col.canFilter) {
                if (col.field) {
                    objControls[col.field] = col.makeFormControl();
                    if (col.disableFilter) {
                        objControls[col.field].disable();
                    }
                }
                else {
                    col.customFilter.forEach((custom) => {
                        objControls[custom.field] = col.makeFormControl(custom.value ? custom.value : null);
                        if (col.disableFilter) {
                            objControls[col.field].disable();
                        }
                    });
                }
            }
        });
        if (Object.keys(objControls).length === 0)
            objControls['none'] = this.fb.control(null);
        this.frmFilter = this.fb.group(objControls);
        if (this.parent && this.parent.loading)
            this.frmFilter.disable();
    }
    enableForm() {
        this.cols.forEach((col) => {
            if (col.canFilter) {
                if (col.field)
                    this.checkFilterStatus(col.field, col);
                else if (col.customFilter)
                    col.customFilter.forEach((custom) => {
                        this.checkFilterStatus(custom.field, col);
                    });
            }
        });
    }
    checkFilterStatus(field, col) {
        let control = this.filterCtrls[field];
        if (control) {
            if (col.disableFilter)
                control.disable();
            else
                control.enable();
        }
    }
    checkDefaultValue(field, value, col) {
        let control = this.filterCtrls[field];
        if (control && value)
            control.setValue(col.filterValue);
    }
    onRender(parent) {
        this.parent = parent;
        this.configureChildrens();
        this.queryCols.changes.subscribe(this.configureChildrens.bind(this));
        this.onBodyWidthChange(this.parent.lastBodyWidth);
    }
    configureChildrens() {
        this.makeForm();
        this.cols.forEach((col, index) => {
            col.onRender(this, index);
            if (col.field)
                this.checkDefaultValue(col.field, col.filterValue, col);
            else if (col.customFilter)
                col.customFilter.forEach((custom) => {
                    this.checkDefaultValue(custom.field, custom.value, col);
                });
        });
        if (this.colSorting) {
            if (this.parent.initialized)
                this.parent.onSortChange();
        }
        else if (this.cols.some(x => x.sort === 'asc' || x.sort === 'desc')) {
            let colSort = this.cols.find(x => x.sort === 'asc' || x.sort === 'desc');
            this.colSorting = {
                col: colSort,
                asc: colSort.sort === 'asc'
            };
            colSort.sort = null;
            if (this.parent.initialized)
                this.parent.onSortChange();
        }
        if (this.parent)
            this.parent.definePaddingTop();
    }
    getColByField(field) {
        return this.cols.find(col => {
            if (col.field)
                return col.field === field;
            else if (col.customFilter)
                return col.customFilter.some(x => x.field === field);
            return false;
        });
    }
    getColFilterModeByField(field) {
        let col = this.getColByField(field);
        if (col) {
            if (col.field)
                return col.mode;
            else if (col.customFilter) {
                let customFilter = col.customFilter.find(x => x.field === field);
                if (customFilter)
                    return customFilter.mode;
            }
        }
        return AgTableFilterMode.CONTAINS;
    }
    getFormDataModel() {
        let formValue = this.frmFilter.value;
        let dataModel = {};
        for (let field in formValue) {
            let value = formValue[field];
            if (!isNullOrUndefined(value) && value !== '')
                dataModel[field] = value;
        }
        return dataModel;
    }
    onBodyWidthChange(currentWidth) {
        if (currentWidth && this.el) {
            let currentWidthNumber = this.helper.onlyNumberAndToFloat(currentWidth);
            let minWidthIsDefined = !!(this.parent && this.parent.minWidth);
            let headerWidth = this.el.clientWidth;
            if (minWidthIsDefined) {
                this.el.style.width = currentWidth;
                headerWidth = this.parent.el.clientWidth;
            }
            else
                this.el.style.width = '';
            if (this.cols.length > 0) {
                this.cols.forEach(col => col.setWidth());
                let lastCol = this.cols[this.cols.length - 1];
                if (headerWidth > currentWidthNumber) {
                    this.el.style.paddingRight = minWidthIsDefined ? '' : ((headerWidth - currentWidthNumber) + 'px');
                    lastCol.el.style.paddingRight = '0px';
                }
                else {
                    this.el.style.paddingRight = '';
                    lastCol.el.style.paddingRight = '';
                }
            }
        }
    }
    ngOnDestroy() {
    }
};
AgTableHeaderComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Helper },
    { type: FormBuilder }
];
__decorate([
    HostBinding('class.ag-table-header')
], AgTableHeaderComponent.prototype, "class", void 0);
__decorate([
    ContentChildren(AgTableColComponent)
], AgTableHeaderComponent.prototype, "queryCols", void 0);
AgTableHeaderComponent = __decorate([
    Component({
        selector: 'ag-table-header',
        template: "<form [formGroup]=\"frmFilter\">\n    <ng-content></ng-content>\n</form>"
    })
], AgTableHeaderComponent);

const TRANSLATION$3 = {
    EMPTY: {
        ptBR: 'Você não tem dados para serem exibidos.',
        enUS: 'You do not have data to display.'
    }
};

const TRANSLATION$4 = {};

const TRANSLATION$5 = {};

let AgTableCellComponent = class AgTableCellComponent {
    constructor(elRef, helper, detectorRef) {
        this.elRef = elRef;
        this.helper = helper;
        this.detectorRef = detectorRef;
        this.class = true;
        this.noTruncate = false;
        this.cellIndex = null;
        this.destroyed = false;
        this.dictionary = TRANSLATION$5;
        this.truncate = !this.noTruncate;
    }
    get truncate() { return this.el.classList.contains('text-truncate'); }
    set truncate(value) {
        if (this.el) {
            if (value)
                this.el.classList.add('text-truncate');
            else
                this.el.classList.remove('text-truncate');
        }
    }
    get col() {
        if (!isNullOrUndefined(this.cellIndex) &&
            this.parent &&
            this.parent.parent &&
            this.parent.parent.parent &&
            this.parent.parent.parent.header &&
            this.parent.parent.parent.header.cols) {
            if (this.cellIndex < this.parent.parent.parent.header.cols.length)
                return this.parent.parent.parent.header.cols[this.cellIndex];
            else {
                console.error('There are more cells than columns. Please fix this.');
                return null;
            }
        }
        return null;
    }
    get el() { return (this.elRef && this.elRef.nativeElement) ? this.elRef.nativeElement : null; }
    ngOnInit() {
    }
    ngOnChanges(changes) {
        if (changes) {
            if ('noTruncate' in changes) {
                this.noTruncate = (typeof this.noTruncate === 'string' && (this.noTruncate === '' || this.noTruncate === 'true')) ? true : this.noTruncate;
                this.truncate = !this.noTruncate && (!this.parent || !this.parent.noTruncate);
            }
        }
    }
    onRender(parent, index) {
        this.cellIndex = index;
        this.parent = parent;
        this.truncate = !this.noTruncate && (!this.parent || !this.parent.noTruncate);
        this.setWidth();
    }
    setWidth() {
        if (this.elRef && this.el) {
            let newWidth = this.col ? this.col._width : 'auto';
            let newMaxWidth = newWidth;
            if (newWidth && newWidth.endsWith('%'))
                newMaxWidth = ((this.helper.onlyNumberAndToFloat(newWidth) * this.parent.parent.el.clientWidth) / 100) + 'px';
            if (newWidth !== this.el.style.width || newMaxWidth !== this.el.style.maxWidth) {
                this.el.style.width = newWidth;
                this.el.style.maxWidth = newMaxWidth;
            }
        }
    }
    ngOnDestroy() {
    }
};
AgTableCellComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Helper },
    { type: ChangeDetectorRef }
];
__decorate([
    HostBinding('class.ag-table-cell')
], AgTableCellComponent.prototype, "class", void 0);
__decorate([
    Input('no-truncate')
], AgTableCellComponent.prototype, "noTruncate", void 0);
AgTableCellComponent = __decorate([
    Component({
        selector: 'ag-table-cell',
        template: "<ng-content></ng-content>"
    })
], AgTableCellComponent);

let AgTableRowComponent = class AgTableRowComponent {
    constructor(elRef, viewContainer) {
        this.elRef = elRef;
        this.viewContainer = viewContainer;
        this.class = true;
        this.noTruncate = false;
        this.dictionary = TRANSLATION$4;
    }
    set _height(value) {
        if (this.el) {
            this.el.style.minHeight = value;
            this.el.style.height = value;
        }
    }
    get _height() { return (this.el) ? this.el.style.minHeight : 'auto'; }
    get cells() { return this.queryCells ? this.queryCells.toArray() : []; }
    get el() { return (this.elRef && this.elRef.nativeElement) ? this.elRef.nativeElement : null; }
    ngOnInit() {
    }
    ngOnChanges(changes) {
        if (changes) {
            if ('noTruncate' in changes) {
                this.noTruncate = (typeof this.noTruncate === 'string' && (this.noTruncate === '' || this.noTruncate === 'true')) ? true : this.noTruncate;
                if (this.cells.length)
                    this.cells.forEach(cell => {
                        cell.truncate = !cell.noTruncate && !this.noTruncate;
                    });
            }
        }
    }
    ngAfterViewInit() {
    }
    onRender(parent, virtualIndex) {
        this.parent = parent;
        this.virtualIndex = virtualIndex;
        this.index = this.parent.getRowIndexBasedVirtual(virtualIndex);
        this._height = this.parent.rowHeight;
        this.configureChildrens();
        this.defineClass();
        if (this.subscription)
            this.subscription.unsubscribe();
        this.subscription = this.queryCells.changes.subscribe(this.configureChildrens.bind(this));
    }
    configureChildrens() {
        this.cells.forEach((cell, index) => {
            cell.onRender(this, index);
        });
    }
    defineClass() {
        if (this.index >= 0) {
            let position = this.index + 1;
            let currentClassName = 'ag-table-row-' + ((position % 2) === 0 ? 'even' : 'odd');
            let previousClassName = 'ag-table-row-' + ((position % 2) === 0 ? 'odd' : 'even');
            if (!this.el.classList.contains(currentClassName))
                this.el.classList.add(currentClassName);
            if (this.el.classList.contains(previousClassName))
                this.el.classList.remove(previousClassName);
        }
    }
    ngOnDestroy() {
        if (this.subscription)
            this.subscription.unsubscribe();
    }
};
AgTableRowComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: ViewContainerRef }
];
__decorate([
    HostBinding('class.ag-table-row')
], AgTableRowComponent.prototype, "class", void 0);
__decorate([
    ContentChildren(AgTableCellComponent)
], AgTableRowComponent.prototype, "queryCells", void 0);
__decorate([
    Input('no-truncate')
], AgTableRowComponent.prototype, "noTruncate", void 0);
AgTableRowComponent = __decorate([
    Component({
        selector: 'ag-table-row',
        template: "<ng-content></ng-content>"
    })
], AgTableRowComponent);

class AgTableVirtualScrollModel {
    constructor(obj) {
        this.currentStartIndex = 0;
        this.currentEndIndex = 0;
        this.totalHeightContent = 0;
        this.paddingScroll = 0;
        this.previousItemsHeight = [];
        if (obj)
            Object.assign(this, obj);
    }
    get currentSrollTop() { return (this.containerSrollEl && this.containerSrollEl.nativeElement) ? this.containerSrollEl.nativeElement.scrollTop : 0; }
    get scrollHeight() { return (this.containerSrollEl && this.containerSrollEl.nativeElement) ? this.containerSrollEl.nativeElement.scrollHeight : 0; }
    get height() { return (this.containerSrollEl && this.containerSrollEl.nativeElement) ? this.containerSrollEl.nativeElement.clientHeight : 0; }
}

class AgTableDataRenderEvent {
    constructor(obj) {
        this.items = [];
        this.length = 0;
        this.startIndex = 0;
        this.endIndex = 0;
        Object.assign(this, obj);
    }
}

let AgTableVirtualScrollService = class AgTableVirtualScrollService {
    constructor(helper) {
        this.helper = helper;
    }
    defineItens(body) {
        let vs = body.virtualScroll;
        let table = body.parent;
        let dimensions = this.defineDimensions(body);
        vs.paddingScroll = dimensions.paddingTop;
        vs.totalHeightContent = dimensions.heightAllItems;
        vs.currentStartIndex = dimensions.itemsThatAreGone;
        vs.currentEndIndex = vs.currentStartIndex + this.howManyCanAppear(body);
        table.items = table.filteredItems.slice(vs.currentStartIndex, vs.currentEndIndex);
        table.onDataRender.emit(new AgTableDataRenderEvent({
            items: table.items,
            length: table.items.length,
            startIndex: vs.currentStartIndex,
            endIndex: vs.currentEndIndex
        }));
        table.isDataEmpty = table.items.length === 0;
    }
    onScrollChange(body) {
        let vs = body.virtualScroll;
        for (let i = 0; i < body.itemsContainerEl.nativeElement.children.length; i++) {
            let children = body.itemsContainerEl.nativeElement.children[i];
            let realIndex = vs.currentStartIndex + i;
            if (!vs.previousItemsHeight[realIndex])
                vs.previousItemsHeight[realIndex] = children.getBoundingClientRect().height;
        }
        this.defineItens(body);
    }
    reset(body) {
        let vs = body.virtualScroll;
        vs.previousItemsHeight = [];
        vs.totalHeightContent = 0;
        vs.paddingScroll = 0;
    }
    defineDimensions(body) {
        let vs = body.virtualScroll;
        let rowHeight = parseInt(this.helper.onlyNumbers(body.rowHeight));
        let obj = {
            paddingTop: 0,
            itemsThatAreGone: 0,
            heightAllItems: 0
        };
        obj.heightAllItems = body.parent.filteredItems.reduce((prev, curr, i) => {
            let height = vs.previousItemsHeight[i];
            return prev + (height ? height : rowHeight);
        }, 0);
        if (vs.currentSrollTop >= rowHeight) {
            let paddingTop = 0;
            let itemsThatAreGone = 0;
            let initialScroll = vs.currentSrollTop;
            for (let h of vs.previousItemsHeight) {
                let height = h ? h : rowHeight;
                if (initialScroll >= height) {
                    paddingTop += height;
                    initialScroll -= height;
                    itemsThatAreGone++;
                }
                else
                    break;
            }
            obj.paddingTop = paddingTop;
            obj.itemsThatAreGone = itemsThatAreGone;
        }
        return obj;
    }
    preparePreviousItemAfterDataChange(body) {
        let table = body.parent;
        let vs = body.virtualScroll;
        if (!table.infinity)
            vs.previousItemsHeight = new Array(table.filteredItems.length).fill(null);
        else {
            let anothers = table.filteredItems.length - vs.previousItemsHeight.length;
            if (anothers > 0)
                vs.previousItemsHeight = [...vs.previousItemsHeight, ...(new Array(anothers).fill(null))];
        }
    }
    canApplyVirtualScroll(body) {
        let vs = body.virtualScroll;
        if (body.parent && body.parent._height !== 'auto')
            return true;
        else {
            let rowHeight = parseInt(this.helper.onlyNumbers(body.rowHeight));
            let avaiableScrollHeight = vs.scrollHeight - vs.height;
            if (avaiableScrollHeight > rowHeight)
                return (avaiableScrollHeight / rowHeight) >= 1.5;
        }
        return false;
    }
    howManyCanAppear(body) {
        let vs = body.virtualScroll;
        let rowHeight = parseInt(this.helper.onlyNumbers(body.rowHeight));
        let height = vs.height ? vs.height : parseInt(this.helper.onlyNumbers(body.parent._height));
        return Math.floor((isNaN(height) ? 0 : height) / rowHeight) + 1;
    }
};
AgTableVirtualScrollService.ctorParameters = () => [
    { type: Helper }
];
AgTableVirtualScrollService = __decorate([
    Injectable()
], AgTableVirtualScrollService);

let AgTableBodyComponent = class AgTableBodyComponent {
    constructor(langService, renderer, helper, elRef, dataVirtualScrollService) {
        this.langService = langService;
        this.renderer = renderer;
        this.helper = helper;
        this.elRef = elRef;
        this.dataVirtualScrollService = dataVirtualScrollService;
        this.class = true;
        this.clickable = false;
        this.minHeight = 40;
        this.rowHeight = this.minHeight + 'px';
        this.emptyMsg = '';
        this.dictionary = TRANSLATION$3;
        this.scrollInBottom = false;
        this.scrollEnabled = false;
        this.lastScrollTop = -1;
        this.eventsLineters = [];
    }
    get heightIsAuto() { return this.parent && this.parent.height === 'auto'; }
    get rows() { return this.queryRows ? this.queryRows.toArray() : []; }
    get el() { return (this.elRef && this.elRef.nativeElement) ? this.elRef.nativeElement : null; }
    ngOnInit() {
        this.virtualScroll = new AgTableVirtualScrollModel({
            containerSrollEl: this.elRef
        });
    }
    ngAfterViewInit() {
        if (!this.emptyMsg)
            this.emptyMsg = this.langService.getText('EMPTY', this.dictionary);
    }
    ngOnChanges(changes) {
        if ('rowHeight' in changes) {
            if (this.helper.onlyNumbers(this.rowHeight) === this.rowHeight)
                this.rowHeight += 'px';
            if (isNullOrUndefined(this.rowHeight) || this.rowHeight === '') {
                this.rowHeight = this.minHeight + 'px';
                return;
            }
            else {
                let number = this.helper.onlyNumbers(this.rowHeight);
                let unit = this.rowHeight.replace(number, '');
                if (unit !== 'px') {
                    this.rowHeight = this.minHeight + 'px';
                    console.warn(`The row-height is invalid. The height must be informed in "px".`);
                    return;
                }
            }
            let number = parseInt(this.helper.onlyNumbers(this.rowHeight));
            if (number < this.minHeight) {
                this.rowHeight = this.minHeight + 'px';
                console.warn(`The row-height is invalid because the minimum height should be "${this.minHeight}px".`);
                return;
            }
        }
    }
    onRender(parent) {
        this.parent = parent;
        this.configureChildrens();
        this.applyListeners();
        this.scrollEnabled = this.parent.loading;
        this.parent.checkMinWidthcanApply();
        if (this.dataVirtualScrollService.canApplyVirtualScroll(this))
            this.dataVirtualScrollService.defineItens(this);
        else
            this.parent.items = this.parent.filteredItems;
        if (this.subscription)
            this.subscription.unsubscribe();
        this.subscription = this.queryRows.changes.subscribe(this.configureChildrens.bind(this));
    }
    configureChildrens() {
        this.clickable = this.parent.clickable;
        this.rows.forEach((row, index) => {
            row.onRender(this, index);
        });
    }
    applyListeners() {
        if (this.eventsLineters && this.eventsLineters.length) {
            this.eventsLineters.forEach((eventListener) => eventListener && eventListener());
            this.eventsLineters = [];
        }
        this.eventsLineters.push(this.renderer.listen(this.elRef.nativeElement, 'scroll', this.onScroll.bind(this)));
        this.eventsLineters.push(this.renderer.listen(this.elRef.nativeElement, 'mousewheel', this.onMouseWheel.bind(this)));
        this.eventsLineters.push(this.renderer.listen(this.elRef.nativeElement, 'DOMMouseScroll', this.onMouseWheel.bind(this)));
    }
    dataTableInBottom() {
        let element = this.elRef.nativeElement;
        return element ? ((element.clientHeight + element.scrollTop) >= element.scrollHeight) : false;
    }
    onScrollFinished() {
        if (this.parent.infinity && this.parent && !this.parent.dataAreOver) {
            this.parent.currentPage++;
            this.parent.emitGetData();
        }
    }
    onScroll(ev, forceChange = false) {
        if (!this.parent.loading) {
            if (this.dataTableInBottom()) {
                if (!this.scrollInBottom) {
                    this.scrollInBottom = true;
                    this.onScrollFinished();
                }
            }
            else
                this.scrollInBottom = false;
        }
        if (this.dataVirtualScrollService.canApplyVirtualScroll(this))
            if (forceChange || this.lastScrollTop !== this.virtualScroll.currentSrollTop) {
                this.lastScrollTop = this.virtualScroll.currentSrollTop;
                this.dataVirtualScrollService.onScrollChange(this);
            }
    }
    backToTheTop() {
        this.el.scrollTop = 0;
    }
    getRowIndexBasedVirtual(virtualIndex) {
        return this.virtualScroll.currentStartIndex + virtualIndex;
    }
    onMouseWheel() {
        if (!this.dataVirtualScrollService.canApplyVirtualScroll(this))
            this.onScroll();
    }
    onBodyWidthChange(currentWidth) {
        if (currentWidth && this.el) {
            if (this.rows.length)
                this.rows.forEach(row => {
                    row.cells.forEach(cell => cell.setWidth());
                });
        }
    }
    ngOnDestroy() {
        if (this.subscription)
            this.subscription.unsubscribe();
        if (this.eventsLineters && this.eventsLineters.length) {
            this.eventsLineters.forEach((eventListener) => eventListener && eventListener());
            this.eventsLineters = [];
        }
    }
};
AgTableBodyComponent.ctorParameters = () => [
    { type: AgTableLangService },
    { type: Renderer2 },
    { type: Helper },
    { type: ElementRef },
    { type: AgTableVirtualScrollService }
];
__decorate([
    HostBinding('class.ag-table-body')
], AgTableBodyComponent.prototype, "class", void 0);
__decorate([
    HostBinding('class.clickable')
], AgTableBodyComponent.prototype, "clickable", void 0);
__decorate([
    HostBinding('class.height-is-auto')
], AgTableBodyComponent.prototype, "heightIsAuto", null);
__decorate([
    ViewChild('itemsContainer', { static: true })
], AgTableBodyComponent.prototype, "itemsContainerEl", void 0);
__decorate([
    ContentChildren(AgTableRowComponent)
], AgTableBodyComponent.prototype, "queryRows", void 0);
__decorate([
    Input('row-height')
], AgTableBodyComponent.prototype, "rowHeight", void 0);
__decorate([
    Input('empty-msg')
], AgTableBodyComponent.prototype, "emptyMsg", void 0);
AgTableBodyComponent = __decorate([
    Component({
        selector: 'ag-table-body',
        template: "\n<section [style.height]=\"parent?.items.length ? '100%' : 'auto'\">\n    <div class=\"total-padding\" [style.height.px]=\"virtualScroll.totalHeightContent\"></div>\n    <div #itemsContainer class=\"scrollable-content\"\n        [ngClass]=\"{ 'p-relative': parent?.height === 'auto' }\"\n        [style.transform]=\"'translateY(' + virtualScroll.paddingScroll + 'px)'\"\n        [style.top.px]=\"(parent?.infinity && parent?.loading && parent?.currentPage > 1) ? -80 : 0\"\n    >\n        <ng-content></ng-content>\n    </div>\n</section>\n\n<div class=\"ag-table-empty animated fadeIn\" *ngIf=\"(parent && !parent.loading && !parent.noEmptyView) && rows && rows.length <= 0\">\n    <span [innerHTML]=\"emptyMsg\"></span>   \n</div>\n\n<div *ngIf=\"parent?.infinity\" class=\"ag-table-infinity\" [ngClass]=\"{'ag-table-infinity-show': parent?.loading && parent?.currentPage > 1}\" [style.width.px]=\"itemsContainer.clientWidth\">\n    <div>\n        <ag-table-spinner></ag-table-spinner>\n    </div>\n</div>"
    })
], AgTableBodyComponent);

var AgTableChangeAction;
(function (AgTableChangeAction) {
    AgTableChangeAction["INITIALIZE"] = "initialization";
    AgTableChangeAction["FILTER"] = "filter";
    AgTableChangeAction["ORDER"] = "order";
    AgTableChangeAction["PAGINATE"] = "paginate";
})(AgTableChangeAction || (AgTableChangeAction = {}));

class AgTableEvent {
    constructor(obj) {
        this.page = 1;
        this.pageSize = 0;
        this.order = {};
        this.resetData = false;
        Object.assign(this, obj);
    }
}

let AgTablePrepareService = class AgTablePrepareService {
    constructor(helper) {
        this.helper = helper;
    }
    apply(data, AgTable) {
        if (data && isArray(data) && data.length) {
            if (!AgTable.serverSide) {
                let _data;
                let length = data.length;
                let lengthPaginated = data.length;
                if (AgTable.isFiltering) {
                    _data = this.applyFilter(data, AgTable);
                    length = _data.length;
                }
                if (AgTable.isSorting) {
                    _data = this.applySort(_data ? _data : data, AgTable);
                    length = _data.length;
                }
                if (AgTable.isPaging) {
                    _data = this.applyPaginate(_data ? _data : data, AgTable);
                    lengthPaginated = _data.length;
                }
                AgTable.setDataLength(length, lengthPaginated);
                return _data ? _data : data;
            }
            return data;
        }
        else {
            if (!AgTable.serverSide)
                AgTable.setDataLength(0, 0);
            return [];
        }
    }
    applyFilter(data, table) {
        let header = table.header;
        let frm = header.frmFilter;
        if (table.actionChange === AgTableChangeAction.FILTER)
            table.setCurrentPage(1);
        let _data = data.filter((item) => {
            let ok = true;
            for (let field in frm.controls) {
                let value;
                if (field.includes('.')) {
                    let fields = field.split('.');
                    value = item[fields[0]];
                    fields.slice(1).forEach(field => {
                        if (isObject(value))
                            value = (field in value) ? value[field] : null;
                    });
                }
                else
                    value = item[field];
                let filterCol = header.getColByField(field);
                let filterMode = header.getColFilterModeByField(field);
                let filterValue = frm.controls[field].value;
                if (filterCol && filterCol.filter === AgTableFilterType.DATE) {
                    value = this.helper.dateFormat(value, filterCol.dateFormat);
                    filterValue = filterValue;
                }
                else {
                    if (filterValue || (typeof filterValue === 'number') || (typeof filterValue === 'boolean'))
                        filterValue = this.helper.removeAccents(filterValue.toString()).toUpperCase();
                    else
                        filterValue = '';
                    if (value || (typeof value === 'number') || (typeof value === 'boolean'))
                        value = this.helper.removeAccents(value.toString()).toUpperCase();
                    else
                        value = '';
                }
                if (filterCol && !isNullOrUndefined(filterValue) && filterValue !== '') {
                    filterValue = this.helper.removeAccents(filterValue.toString()).toUpperCase();
                    switch (filterMode) {
                        case AgTableFilterMode.CONTAINS:
                            ok = value.includes(filterValue);
                            break;
                        case AgTableFilterMode.EQUALS:
                            ok = value === filterValue;
                            break;
                        case AgTableFilterMode.STARTS_WITH:
                            ok = value.startsWith(filterValue);
                            break;
                        case AgTableFilterMode.ENDS_WITH:
                            ok = value.endsWith(filterValue);
                            break;
                    }
                }
                if (!ok)
                    break;
            }
            return ok;
        });
        return _data;
    }
    applySort(data, table) {
        if (table.actionChange === AgTableChangeAction.ORDER)
            table.setCurrentPage(1);
        if (table.header.colSorting) {
            let col = table.header.colSorting.col;
            let asc = table.header.colSorting.asc;
            let field = '';
            if (!col.field && col.customFilter && col.customFilter.length)
                field = col.customFilter[0].field;
            else
                field = col.field;
            let dateRegex = new RegExp('^(-?(?:[1-9][0-9]*)?[0-9]{4})-(1[0-2]|0[1-9])-(3[01]|0[1-9]|[12][0-9])T(2[0-3]|[01][0-9]):([0-5][0-9]):([0-5][0-9])(.[0-9]+)?(Z)?$');
            let _data = data.sort((a, b) => {
                let valueA = a[field];
                let valueB = b[field];
                if (typeof valueA === 'string' && dateRegex.test(valueA))
                    valueA = new Date(valueA);
                if (typeof valueB === 'string' && dateRegex.test(valueB))
                    valueB = new Date(valueB);
                if (typeof valueA === 'string' && !isNaN(parseFloat(valueA)))
                    valueA = parseFloat(valueA);
                if (typeof valueB === 'string' && !isNaN(parseFloat(valueB)))
                    valueB = parseFloat(valueB);
                if (valueA < valueB)
                    return asc ? -1 : 1;
                if (valueA > valueB)
                    return asc ? 1 : -1;
                return 0;
            });
            return _data;
        }
        return data;
    }
    applyPaginate(data, table) {
        let currentPage = table.currentPage;
        let size = table.paginate;
        let begin = (currentPage - 1) * size;
        let end = begin + size;
        let _data = data.slice(begin, end);
        return _data;
    }
};
AgTablePrepareService.ctorParameters = () => [
    { type: Helper }
];
AgTablePrepareService = __decorate([
    Injectable()
], AgTablePrepareService);

const TRANSLATION$6 = {
    TOTAL_CAPTION: {
        ptBR: 'Exibindo de {{start}} até {{end}} de {{total}} .',
        enUS: 'Displaying from {{start}} to {{end}} of {{total}} .'
    },
    WITHOUT_TOTAL_CAPTION: {
        ptBR: 'Exibindo de {{start}} até {{end}} registros.',
        enUS: 'Displaying from {{start}} to {{end}} records.'
    }
};

let AgTablePaginateComponent = class AgTablePaginateComponent {
    constructor(elRef) {
        this.elRef = elRef;
        this.class = true;
        this.length = 0;
        this.current = 1;
        this.disabled = false;
        this.loading = false;
        this.change = new EventEmitter();
        this.dictionary = TRANSLATION$6;
        this.pages = [];
    }
    get el() { return (this.elRef && this.elRef.nativeElement) ? this.elRef.nativeElement : null; }
    ngOnInit() {
    }
    onRender(parent) {
        this.parent = parent;
        this.parent.checkMinWidthcanApply();
    }
    ngOnChanges(changes) {
        if ('length' in changes || 'current' in changes)
            this.generate();
    }
    pageClick(page) {
        if (!this.disabled && (typeof page) !== 'string' && page >= 1 && page <= this.length && page !== this.current) {
            this.current = page;
            this.change.emit(this.current);
            this.generate();
        }
    }
    generate() {
        let current = this.current, last = this.length, delta = 2, left = current - delta, right = current + delta + 1, range = [], rangeWithDots = [], l;
        if (this.current > 0 && this.length > 0) {
            for (let i = 1; i <= last; i++) {
                if (i === 1 || i === last || i >= left && i < right)
                    range.push(i);
            }
            for (let i of range) {
                if (l) {
                    if (i - l === 2)
                        rangeWithDots.push(l + 1);
                    else if (i - l !== 1)
                        rangeWithDots.push('...');
                }
                rangeWithDots.push(i);
                l = i;
            }
        }
        this.pages = rangeWithDots;
    }
};
AgTablePaginateComponent.ctorParameters = () => [
    { type: ElementRef }
];
__decorate([
    HostBinding('class.ag-table-paginate')
], AgTablePaginateComponent.prototype, "class", void 0);
__decorate([
    Input()
], AgTablePaginateComponent.prototype, "length", void 0);
__decorate([
    Input()
], AgTablePaginateComponent.prototype, "current", void 0);
__decorate([
    Input()
], AgTablePaginateComponent.prototype, "disabled", void 0);
__decorate([
    Input()
], AgTablePaginateComponent.prototype, "loading", void 0);
__decorate([
    Input()
], AgTablePaginateComponent.prototype, "objCaption", void 0);
__decorate([
    Output()
], AgTablePaginateComponent.prototype, "change", void 0);
AgTablePaginateComponent = __decorate([
    Component({
        selector: 'ag-table-paginate',
        template: "<span *ngIf=\"objCaption && !loading\" class=\"caption animated fadeIn\" [ngClass]=\"{ 'p-b-7': (length <= 1 || pages.length === 0) }\">\n\t{{ (objCaption.total ? 'TOTAL_CAPTION' : 'WITHOUT_TOTAL_CAPTION') | translate:dictionary:objCaption }}\n</span>\n\n<ul *ngIf=\"length > 1 && pages.length > 0\">\n\t<li [ngClass]=\"{'disabled': current <= 1}\" class=\"animated fadeIn\">\n\t\t<button (click)=\"pageClick(current - 1)\">\n\t\t\t<i class=\"arrow-prev\"></i>\n\t\t</button>\n\t</li>\n\t<li *ngFor=\"let page of pages\" class=\"animated fadeIn\"\n\t\t[ngClass]=\"{'actived': page === current, 'disabled': disabled, 'interval': page === '...'}\">\n\t\t<button (click)=\"pageClick(page)\">\n\t\t\t<span>{{page}}</span>\n\t\t</button>\n\t</li>\n\t<li [ngClass]=\"{'disabled': current >= length}\" class=\"animated fadeIn\">\n\t\t<button (click)=\"pageClick(current + 1)\">\n\t\t\t<i class=\"arrow-next\"></i>\n\t\t</button>\n\t</li>\n</ul>"
    })
], AgTablePaginateComponent);

let AgTableComponent = class AgTableComponent {
    constructor(elRef, helper, dataPrepareService, dataVirtualScrollService) {
        this.elRef = elRef;
        this.helper = helper;
        this.dataPrepareService = dataPrepareService;
        this.dataVirtualScrollService = dataVirtualScrollService;
        this.class = true;
        /** THAT WILL DEFINE THE DATA TABLE HEIGHT, AND IT WILL HAS SCROLL BAR IN CASE OF OVERFLOW.*/
        this.height = 'auto';
        /** THIS DEFINE IF WILL GO PAGINATE OR NOT */
        this.paginate = null;
        /** THIS DEFINE IF WILL GO BRING DATA BASED ON SCROLLING WHEN TO ARRIVE ON BOTTOM. (BUT USE THIS ONLY WITH SERVER-SIDE) */
        this.infinity = null;
        /** SHOW LOADING SPINNER ABOVE THE DATA TABLE WHEN LOADING IS TRUE. */
        this.loading = null;
        /** THE TOTAL DATA LENGTH. THIS IS USED FOR MAKE THE PAGINATION. */
        this.dataLength = 0;
        /** DETERMINES WHETHER DATA WILL BE SWALLOWED VIA SERVER-SIDE */
        this.serverSide = false;
        /** CALL REQUEST DATA ON INITIALIZATION DATA TABLE */
        this.getDataOnInit = false;
        /** SET THE CURRENT PAGE WHEN USE PAGINATION */
        this.currentPage = 1;
        /** NOTIFIES THAT THE DATA IS GONE (INFINITE-SCROLL ONLY). */
        this.dataAreOver = false;
        /** SETS WHETHER TO DISPLAY EMPTY-VIEW OR NOT. */
        this.noEmptyView = false;
        /** SETS A MINIMUM HEIGHT IN AG-TABLE. */
        this.minHeight = '';
        /** SETS A MINIMUM WIDTH IN AG-TABLE. WHEN THE MINIMUM WIDTH IS REACHED, THE SIDE TABLE WILL SCROLL HORIZONTALLY. */
        this.minWidth = '';
        this.allItems = [];
        /** CALL/EMIT EVENT TO GET DATA (SERVER-SIDE) */
        this.onGetData = new EventEmitter();
        /** EMIT EVENT WICH ITEMS HAVEN BEEN RENDERED */
        this.onDataRender = new EventEmitter();
        /** EMIT EVENT WHEN THE ag-table HAS ALL BEEN RENDERED */
        this.onTableRender = new EventEmitter();
        this.filterActivated = new EventEmitter();
        this.items = [];
        this.filteredItems = [];
        this.dictionary = TRANSLATION;
        this.initialized = false;
        this.isDataEmpty = false;
        this.dataPaginatedLength = 0;
        this.actionChange = AgTableChangeAction.INITIALIZE;
        this.DOMisVisible = false;
        this.DOMcountVisibleChange = 0;
        this.lastBodyWidth = null;
        this.lastBodyHeight = null;
    }
    get showEmptyView() { return this.isDataEmpty && !this.noEmptyView; }
    get paddingTop() { return this.elRef && this.elRef.nativeElement ? this.elRef.nativeElement.style.paddingTop : '0px'; }
    set paddingTop(value) { if (this.elRef && this.elRef.nativeElement)
        this.elRef.nativeElement.style.paddingTop = value; }
    set _height(value) {
        this.el.style.height = value;
        if (this.header) {
            if (value === '0' || value === '0px')
                this.header.visible = false;
            else
                this.header.visible = true;
        }
        this.definePaddingTop();
    }
    get _height() { return (this.el && this.el.style.height) ? this.el.style.height : 'auto'; }
    set _minHeight(value) {
        this.el.style.minHeight = value;
        this.definePaddingTop();
    }
    get _minHeight() { return (this.el && this.el.style.minHeight) ? this.el.style.minHeight : ''; }
    get header() { return this.queryHeader ? this.queryHeader.first : null; }
    get body() { return this.queryBody ? this.queryBody.first : null; }
    get topShadow() {
        if (this.height !== 'auto') {
            let body = (this.body) ? this.body.el : null;
            let header = (this.header) ? this.header.el : null;
            if (body && header)
                return body.scrollTop >= (header.clientHeight / 2);
        }
        return false;
    }
    get bottomShadow() {
        if (this.height !== 'auto') {
            let body = (this.body) ? this.body.el : null;
            let header = (this.header) ? this.header.el : null;
            if (body && header)
                return (body.scrollTop + body.clientHeight) < body.scrollHeight;
        }
        return false;
    }
    get isPaging() { return this.paginate; }
    get isSorting() { return this.header && this.header.colSorting; }
    get isFiltering() { return ((this.header && this.header.cols) ? this.header.cols.some(x => x.canFilter) : false); }
    get numPages() {
        if (this.dataLength && this.isPaging) {
            let pages = Math.floor(this.dataLength / this.paginate);
            return pages + (((this.dataLength / this.paginate) > pages) ? 1 : 0);
        }
        else
            return 0;
    }
    get el() { return (this.elRef && this.elRef.nativeElement) ? this.elRef.nativeElement : null; }
    ngOnInit() {
    }
    ngAfterViewInit() {
        setTimeout(() => {
            if (this.body.el) {
                this.lastBodyWidth = this.body.el.clientWidth + 'px';
                this.lastBodyHeight = this.body.el.clientHeight + 'px';
            }
            this.body.onRender(this);
            this.header.onRender(this);
            if (this.paginateComp)
                this.paginateComp.onRender(this);
            this.queryHeader.changes.subscribe(() => this.definePaddingTop());
            this.definePaddingTop();
            setTimeout(() => this.onTableRender.emit());
            if (this.getDataOnInit)
                this.emitGetData();
            if (this.infinity && !this.heightIsValid())
                console.warn(`The infinity scroll mode not working, because data table height has not been set.`);
            if (!this.infinity && this.body)
                this.body.backToTheTop();
            this.filterActivated.emit(false);
            this.initialized = true;
        });
    }
    ngAfterContentChecked() {
        if (this.elRef && this.elRef.nativeElement) {
            let elem = this.elRef.nativeElement;
            let visible = !!(elem.offsetWidth || elem.offsetHeight || elem.getClientRects().length);
            if (visible !== this.DOMisVisible) {
                this.DOMisVisible = visible;
                if (this.DOMisVisible && this.DOMcountVisibleChange > 0)
                    this.body.onScroll();
                this.DOMcountVisibleChange++;
            }
        }
        else
            this.DOMisVisible = false;
        this.onCheckBodyDimensions();
    }
    ngOnChanges(changes) {
        setTimeout(() => {
            if ('serverSide' in changes)
                this.serverSide = (typeof this.serverSide === 'string' && (this.serverSide === '' || this.serverSide === 'true')) ? true : this.serverSide;
            if ('noEmptyView' in changes)
                this.noEmptyView = (typeof this.noEmptyView === 'string' && (this.noEmptyView === '' || this.noEmptyView === 'true')) ? true : this.noEmptyView;
            if ('clickable' in changes) {
                this.clickable = (typeof this.clickable === 'string' && (this.clickable === '' || this.clickable === 'true')) ? true : this.clickable;
                if (this.body)
                    this.body.clickable = this.clickable;
            }
            if ('height' in changes) {
                if (this.height !== 'auto' && !this.heightIsValid()) {
                    console.warn(`The height of table is invalid.`);
                    this.height = 'auto';
                }
                this._height = this.height;
                this.prepareItemsPagingAndFilter();
                if (!this.infinity && this.body)
                    this.body.backToTheTop();
            }
            if ('minHeight' in changes) {
                if (this.minHeight !== 'auto' && !this.minHeightIsValid()) {
                    console.warn(`The minHeight of table is invalid.`);
                    this.minHeight = '';
                }
                this._minHeight = this.minHeight;
                this.prepareItemsPagingAndFilter();
                if (!this.infinity && this.body)
                    this.body.backToTheTop();
            }
            if ('minWidth' in changes) {
                if (!this.minWidthIsValid()) {
                    console.warn(`The min-width of table is invalid.`);
                    this.minWidth = '';
                }
                if (!this.minWidthIsValid())
                    this.stopListenerWidth();
                else
                    this.startListenerWidth();
            }
            if ('currentPage' in changes) {
                if (typeof this.currentPage === 'string') {
                    if (!isNaN(parseInt(this.currentPage)))
                        this.currentPage = parseInt(this.currentPage);
                    else {
                        console.warn(`The current-page value of table is invalid. The current-page value has been defined as 1.`);
                        this.currentPage = 1;
                    }
                }
                if (typeof this.currentPage !== 'number' || this.currentPage <= 0) {
                    console.warn(`The current-page value of table is invalid. The current-page value has been defined as 1.`);
                    this.currentPage = 1;
                }
                if (this.currentPage > this.paginate && this.currentPage < 1)
                    this.currentPage = 1;
                this.onPageChange(this.currentPage, !this.serverSide);
            }
            if ('paginate' in changes) {
                if (typeof this.paginate === 'string') {
                    if (!isNaN(parseInt(this.paginate)))
                        this.paginate = parseInt(this.paginate);
                    else {
                        console.warn(`The [paginate] value of table is invalid. The paginate has not been applied.`);
                        this.paginate = null;
                    }
                }
                if (this.isPaging && this.infinity) {
                    console.warn(`The [paginate] has not been applied because [infinity] already has been defined.`);
                    this.paginate = null;
                }
                if (this.paginate) {
                    if (this.currentPage > this.paginate)
                        this.currentPage = 1;
                    if (this.serverSide)
                        this.updatePaginateConfig();
                }
                this.onPageChange(this.currentPage, !this.serverSide);
                this.definePaddingBottom();
            }
            if ('infinity' in changes) {
                if (typeof this.infinity === 'string') {
                    if (!isNaN(parseInt(this.infinity)))
                        this.infinity = parseInt(this.infinity);
                    else {
                        console.warn(`The [infinity] value of table is invalid. The infinity scroll has not been applied.`);
                        this.infinity = null;
                    }
                }
                if (this.infinity && this.paginate) {
                    console.warn(`The [infinity] has not been applied because [paginate] already has been defined.`);
                    this.infinity = null;
                }
                this.onPageChange(this.currentPage, !this.serverSide);
                this.definePaddingBottom();
            }
            if ('dataLength' in changes) {
                if (isNullOrUndefined(this.dataLength))
                    this.dataLength = 0;
                this.updatePaginateConfig();
                this.definePaddingBottom();
            }
            if ('getDataOnInit' in changes)
                this.getDataOnInit = (typeof this.getDataOnInit === 'string' && (this.getDataOnInit === '' || this.getDataOnInit === 'true')) ? true : this.getDataOnInit;
            if ('loading' in changes) {
                setTimeout(() => {
                    if (this.body)
                        this.body.scrollEnabled = this.loading;
                });
                if (this._height.replace('px', '') === '0' && this.loading)
                    this.elRef.nativeElement.style.height = '50px';
                if (this.header && this.header.frmFilter) {
                    if (this.loading)
                        this.header.frmFilter.disable();
                    else {
                        this.header.enableForm();
                        this.updatePaginateConfig();
                    }
                }
            }
            if ('allItems' in changes) {
                if (!this.infinity && this.currentPage !== 1) {
                    let maxPages = Math.floor(this.dataLength / this.paginate);
                    if ((this.dataLength % this.paginate) !== 0)
                        maxPages++;
                    if (this.currentPage > maxPages || !this.currentPage) {
                        this.currentPage = (this.currentPage > maxPages) ? maxPages : 1;
                        if (this.serverSide) {
                            this.onPageChange(this.currentPage);
                            return;
                        }
                    }
                }
                this.refreshRender();
            }
        }, 100);
    }
    onCheckBodyDimensions() {
        this.checkBodyWidthChange();
        this.checkBodyHeightChange();
    }
    checkBodyWidthChange() {
        if (this.el) {
            let currentWidth = '100%';
            if (this.body && this.body.itemsContainerEl && this.body.itemsContainerEl.nativeElement)
                currentWidth = this.body.itemsContainerEl.nativeElement.clientWidth + 'px';
            if (currentWidth !== this.lastBodyWidth) {
                this.lastBodyWidth = currentWidth;
                this.header.onBodyWidthChange(currentWidth);
                this.body.onBodyWidthChange(currentWidth);
            }
        }
    }
    checkBodyHeightChange() {
        if (this.el && this._height !== 'auto') {
            let currentHeight = this.el.clientHeight + 'px';
            if (currentHeight !== this.lastBodyHeight) {
                this.lastBodyHeight = currentHeight;
                this.refreshRender();
            }
        }
    }
    definePaddingBottom() {
        if (this.el) {
            let padding = '0px';
            if (!this.infinity && this.paginate !== null)
                padding = '46px';
            if (padding !== this.elRef.nativeElement.style.paddingBottom)
                this.el.style.paddingBottom = padding;
        }
    }
    definePaddingTop(forceDefine = false) {
        if (this.el && this.header) {
            let padding = '0px';
            if (this.header && this.header.el && this.header.visible)
                padding = this.header.height;
            if (forceDefine || padding !== this.elRef.nativeElement.style.paddingTop) {
                this.paddingTop = padding;
                if (this.headerShadowEl && this.headerShadowEl.nativeElement)
                    this.headerShadowEl.nativeElement.style.height = padding !== '0px' ? padding : 'auto';
            }
        }
    }
    heightIsValid() {
        return !isNullOrUndefined(this.height) && (typeof this.height === 'string') && this.height !== '' && this.height !== 'auto';
    }
    minHeightIsValid() {
        return !isNullOrUndefined(this.minHeight) && (typeof this.minHeight === 'string') && this.minHeight !== '' && this.minHeight !== 'auto';
    }
    minWidthIsValid() {
        return !isNullOrUndefined(this.minWidth) && (typeof this.minWidth === 'string') && this.minWidth !== '' && this.minWidth !== 'auto';
    }
    startListenerWidth() {
        this.stopListenerWidth();
        this.intervalListenerWidth = setInterval(() => {
            if (this.el && this.minWidthIsValid()) {
                let width = this.el.clientWidth;
                let minWidth = parseInt(this.helper.onlyNumbers(this.minWidth));
                if (width <= minWidth) {
                    this.el.style.overflowX = 'auto';
                    this.el.style.overflowY = 'hidden';
                }
                else {
                    this.el.style.overflowX = '';
                    this.el.style.overflowY = '';
                }
            }
        }, 250);
    }
    stopListenerWidth() {
        if (this.intervalListenerWidth) {
            clearInterval(this.intervalListenerWidth);
            this.intervalListenerWidth = null;
        }
    }
    checkMinWidthcanApply() {
        if (this.minWidthIsValid()) {
            if (this.el)
                this.body.el.style.minWidth = this.minWidth;
            if (this.paginateComp && this.paginateComp.el)
                this.paginateComp.el.style.minWidth = this.minWidth;
        }
        else {
            if (this.body && this.body.el)
                this.body.el.style.minWidth = '';
            if (this.paginateComp && this.paginateComp.el)
                this.paginateComp.el.style.minWidth = '';
        }
    }
    setDataLength(length, paginatedLength) {
        this.dataLength = length;
        this.dataPaginatedLength = paginatedLength;
    }
    onPageChange(currentPage, emit = true) {
        if (this.isPaging) {
            this.notifyChanges(AgTableChangeAction.PAGINATE);
            if (emit && this.serverSide) {
                this.setCurrentPage(currentPage);
                this.emitGetData();
            }
            else if (!this.serverSide) {
                this.setCurrentPage(currentPage, false);
                this.prepareItemsPagingAndFilter();
                this.updatePaginateConfig();
                this.body.backToTheTop();
            }
        }
    }
    onSortChange() {
        this.notifyChanges(AgTableChangeAction.ORDER);
        if (this.serverSide) {
            this.setCurrentPage(1);
            this.emitGetData(this.infinity > 0);
        }
        else {
            this.prepareItemsPagingAndFilter();
            this.body.backToTheTop();
        }
    }
    onFilterChange() {
        this.notifyChanges(AgTableChangeAction.FILTER);
        if (this.header && this.header.cols) {
            if (this.header.cols.some(x => x.filterActive)) {
                this.filterActivated.emit(true);
            }
            else {
                this.filterActivated.emit(false);
            }
        }
        if (this.serverSide) {
            this.setCurrentPage(1);
            this.emitGetData(this.infinity > 0);
        }
        else {
            this.prepareItemsPagingAndFilter();
            this.updatePaginateConfig();
            this.body.backToTheTop();
        }
    }
    prepareItemsPagingAndFilter() {
        this.filteredItems = this.dataPrepareService.apply(this.allItems, this);
        if (this.body && this.dataVirtualScrollService.canApplyVirtualScroll(this.body))
            this.dataVirtualScrollService.defineItens(this.body);
        else {
            this.items = this.filteredItems;
            this.isDataEmpty = this.items.length === 0;
            this.emitDataRender();
        }
    }
    emitGetData(resetData = false) {
        if (resetData) {
            this.currentPage = 1;
            this.dataVirtualScrollService.reset(this.body);
        }
        let colSort = this.header.colSorting;
        this.onGetData.emit(new AgTableEvent({
            page: this.currentPage,
            pageSize: this.paginate ? this.paginate : this.infinity,
            filters: this.header.getFormDataModel(),
            order: colSort ? { field: colSort.col.field, asc: colSort.asc } : null,
            resetData: resetData
        }));
    }
    emitDataRender() {
        this.onDataRender.emit(new AgTableDataRenderEvent({
            items: this.items,
            length: this.items.length,
            startIndex: 0,
            endIndex: this.items.length - 1
        }));
    }
    setCurrentPage(page, updateConfig = true) {
        this.currentPage = page;
        if (updateConfig)
            this.updatePaginateConfig();
    }
    updatePaginateConfig() {
        let visibleLength = this.dataPaginatedLength;
        if (this.serverSide && this.body)
            visibleLength = this.filteredItems.length;
        let page = this.paginate ? this.paginate : this.infinity;
        this.paginateCaptionConfig = {
            start: (page * (this.currentPage - 1)) + 1,
            end: (page * (this.currentPage - 1)) + visibleLength,
            total: this.dataLength ? this.dataLength : 0
        };
    }
    notifyChanges(action = AgTableChangeAction.INITIALIZE) {
        this.actionChange = action;
    }
    resetFilters(getData = false) {
        for (let ctrlName in this.header.filterCtrls) {
            this.header.filterCtrls[ctrlName].reset();
        }
        if (getData)
            this.emitGetData(true);
    }
    refreshRender() {
        if (this.body) {
            if (!this.body.parent)
                this.body.parent = this;
            this.dataVirtualScrollService.preparePreviousItemAfterDataChange(this.body);
        }
        this.prepareItemsPagingAndFilter();
        this.updatePaginateConfig();
        this.definePaddingBottom();
        if (!this.infinity && this.body)
            this.body.backToTheTop();
    }
    ngOnDestroy() {
        this.stopListenerWidth();
    }
};
AgTableComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: Helper },
    { type: AgTablePrepareService },
    { type: AgTableVirtualScrollService }
];
__decorate([
    HostBinding('class.ag-table')
], AgTableComponent.prototype, "class", void 0);
__decorate([
    HostBinding('class.ag-table-empty')
], AgTableComponent.prototype, "showEmptyView", null);
__decorate([
    ContentChildren(AgTableHeaderComponent)
], AgTableComponent.prototype, "queryHeader", void 0);
__decorate([
    ContentChildren(AgTableBodyComponent)
], AgTableComponent.prototype, "queryBody", void 0);
__decorate([
    ViewChild(AgTablePaginateComponent, { static: false })
], AgTableComponent.prototype, "paginateComp", void 0);
__decorate([
    ViewChild('headerShadowEl', { static: true })
], AgTableComponent.prototype, "headerShadowEl", void 0);
__decorate([
    Input()
], AgTableComponent.prototype, "height", void 0);
__decorate([
    Input()
], AgTableComponent.prototype, "clickable", void 0);
__decorate([
    Input()
], AgTableComponent.prototype, "paginate", void 0);
__decorate([
    Input()
], AgTableComponent.prototype, "infinity", void 0);
__decorate([
    Input()
], AgTableComponent.prototype, "loading", void 0);
__decorate([
    Input('data-length')
], AgTableComponent.prototype, "dataLength", void 0);
__decorate([
    Input('server-side')
], AgTableComponent.prototype, "serverSide", void 0);
__decorate([
    Input('get-data-init')
], AgTableComponent.prototype, "getDataOnInit", void 0);
__decorate([
    Input('current-page')
], AgTableComponent.prototype, "currentPage", void 0);
__decorate([
    Input('data-are-over')
], AgTableComponent.prototype, "dataAreOver", void 0);
__decorate([
    Input('no-empty-view')
], AgTableComponent.prototype, "noEmptyView", void 0);
__decorate([
    Input('min-height')
], AgTableComponent.prototype, "minHeight", void 0);
__decorate([
    Input('min-width')
], AgTableComponent.prototype, "minWidth", void 0);
__decorate([
    Input('items')
], AgTableComponent.prototype, "allItems", void 0);
__decorate([
    Output()
], AgTableComponent.prototype, "onGetData", void 0);
__decorate([
    Output()
], AgTableComponent.prototype, "onDataRender", void 0);
__decorate([
    Output()
], AgTableComponent.prototype, "onTableRender", void 0);
__decorate([
    Output()
], AgTableComponent.prototype, "filterActivated", void 0);
AgTableComponent = __decorate([
    Component({
        selector: 'ag-table',
        template: "<div #headerShadowEl class=\"top-shadow animated fadeIn\" [ngClass]=\"{'to-drop': topShadow}\"></div>\n\n<ng-content></ng-content>\n\n<ag-table-paginate *ngIf=\"!infinity && paginate !== null\"\n    class=\"animated fadeIn\"\n    [ngClass]=\"{ 'to-drop': bottomShadow, 'on-infinity': infinity }\"\n    [length]=\"(isPaging && numPages) ? numPages : 1\"\n    [current]=\"(isPaging && numPages) ? currentPage : 1\"\n    [objCaption]=\"(isPaging && numPages) ? paginateCaptionConfig : { start: 1, end: dataLength, total: 0 }\"\n    [loading]=\"loading\"\n    [disabled]=\"loading\"\n    (change)=\"onPageChange($event)\">\n</ag-table-paginate>\n\n<div *ngIf=\"loading && (!infinity || currentPage === 1)\" class=\"ag-table-loading animated fadeIn\">\n    <ag-table-spinner></ag-table-spinner>\n</div>"
    })
], AgTableComponent);

const FILTER_TYPES = Object.keys(AgTableFilterType).map(key => AgTableFilterType[key]);

const TRANSLATION$7 = {
    CURRENCY_PH: {
        ptBR: 'R$',
        enUS: 'USD'
    },
    DATE_FORMAT: {
        ptBR: 'dd/MM/yyyy',
        enUS: 'yyyy/MM/dd'
    },
    YEAR_INITIAL: {
        ptBR: 'a',
        enUS: 'y',
    }
};

let AgTableFilterComponent = class AgTableFilterComponent {
    constructor(helper, langService, settings) {
        this.helper = helper;
        this.langService = langService;
        this.settings = settings;
        this.class = true;
        this.currentFilter = '';
        this.type = '';
        this.placeholder = '';
        this.dateFormat = '';
        this.optionAllLabel = '';
        this.options = [];
        /* DISABLED
        @Input() public readOnly: boolean = false;
        @Input() public minDate: Date;
        @Input() public maxDate: Date;
        @Input() public dateSeparator: string = '/'; */
        this.onChange = new EventEmitter();
        this.onRender = new EventEmitter();
        this.selectAllOptionValue = null;
        this.dictionary = TRANSLATION$7;
    }
    get debounceTime() {
        if (this.type === 'date' || this.type === 'select')
            return 0;
        else
            return this.settings.filterDebounceTime;
    }
    ngOnInit() {
        setTimeout(() => {
            this.onRender.emit();
        });
    }
    ngOnChanges(changes) {
        if ('options' in changes) {
            if (!isArray(this.options)) {
                console.warn('The options of a filter of type "select" was set invalid.');
                this.options = [];
            }
            else if (this.options.length) {
                if (!isObject(this.options[0])) {
                    this.options = this.options.map((x) => {
                        return {
                            text: x,
                            value: x
                        };
                    });
                }
                else {
                    this.options = this.options.map((x) => {
                        return {
                            text: ('text' in x) ? x.text : x.value,
                            value: ('id' in x) ? x.id : x.value
                        };
                    });
                }
            }
        }
        if ('type' in changes) {
            if (FILTER_TYPES.every(x => x !== this.type))
                this.type = AgTableFilterType.TEXT;
        }
        if ('placeholder' in changes) {
            if (!this.placeholder && this.type === 'date')
                this.placeholder = this.dateFormat.replace(/y/g, this.langService.getText('YEAR_INITIAL', this.dictionary)).toUpperCase();
        }
        if ('frmControl' in changes) {
            if (this.subscription)
                this.subscription.unsubscribe();
            if (this.frmControl) {
                this.currentFilter = this.frmControl.value;
                this.subscription = this.frmControl.valueChanges.pipe(debounceTime(this.debounceTime), map(this.prepareData.bind(this))).subscribe((toFilter) => {
                    if (toFilter && this.frmControl.enable)
                        this.onChange.emit(this.currentFilter);
                });
            }
            else
                this.currentFilter = null;
        }
    }
    prepareData() {
        if (this.frmControl.disabled) {
            this.frmControl.disable();
            return false;
        }
        if (this.type === 'select') {
            if (this.frmControl.value === 'undefined' || this.frmControl.value === 'null') {
                this.currentFilter = this.selectAllOptionValue;
                this.frmControl.setValue(this.currentFilter);
                return true;
            }
        }
        if (this.frmControl.value !== this.currentFilter) {
            if (this.type === 'date') {
                if (this.frmControl.value && !this.helper.dateIsValid(this.frmControl.value, this.dateFormat)) {
                    this.currentFilter = this.selectAllOptionValue;
                    this.frmControl.setValue(null);
                    return false;
                }
            }
            this.currentFilter = this.frmControl.value;
            if (typeof this.currentFilter === 'string' && this.currentFilter !== '') {
                if (this.currentFilter === 'null') {
                    this.currentFilter = null;
                    this.frmControl.setValue(this.currentFilter);
                }
                else if (this.currentFilter === 'undefined') {
                    this.currentFilter = undefined;
                    this.frmControl.setValue(this.currentFilter);
                }
                else if (this.helper.onlyNumbers(this.currentFilter) === this.currentFilter) {
                    this.currentFilter = parseInt(this.currentFilter);
                    this.frmControl.setValue(this.currentFilter);
                }
            }
        }
        else
            return false;
        return true;
    }
    isEmptyValue(value) {
        return isNullOrUndefined(value) || value.toString() === '';
    }
    ngOnDestroy() {
        if (this.subscription)
            this.subscription.unsubscribe();
    }
};
AgTableFilterComponent.ctorParameters = () => [
    { type: Helper },
    { type: AgTableLangService },
    { type: AgTableSettings }
];
__decorate([
    HostBinding('class.ag-table-filter')
], AgTableFilterComponent.prototype, "class", void 0);
__decorate([
    Input()
], AgTableFilterComponent.prototype, "type", void 0);
__decorate([
    Input()
], AgTableFilterComponent.prototype, "placeholder", void 0);
__decorate([
    Input()
], AgTableFilterComponent.prototype, "dateFormat", void 0);
__decorate([
    Input()
], AgTableFilterComponent.prototype, "optionAllLabel", void 0);
__decorate([
    Input()
], AgTableFilterComponent.prototype, "options", void 0);
__decorate([
    Input()
], AgTableFilterComponent.prototype, "frmControl", void 0);
__decorate([
    Output()
], AgTableFilterComponent.prototype, "onChange", void 0);
__decorate([
    Output()
], AgTableFilterComponent.prototype, "onRender", void 0);
AgTableFilterComponent = __decorate([
    Component({
        selector: 'ag-table-filter',
        template: "<input *ngIf=\"type === 'currency'\"\n    class=\"ag-table-filter-input\"\n    type=\"text\"\n    autocomplete=\"off\"\n    [formControl]=\"frmControl\"\n    [placeholder]=\"placeholder ? placeholder : (('CURRENCY_PH' | translate:dictionary) + ' 0,00')\"\n/>\n\n<select *ngIf=\"type === 'select'\" \n    class=\"ag-table-filter-select\"\n    [formControl]=\"frmControl\">\n    <option [value]=\"selectAllOptionValue\">{{optionAllLabel}}</option>\n    <option *ngFor=\"let option of options\" [value]=\"option.value\">{{option.text}}</option>\n</select>\n\n<input *ngIf=\"type === 'text'\"\n    class=\"ag-table-filter-input\"\n    [formControl]=\"frmControl\"\n    type=\"text\"\n    [placeholder]=\"placeholder ? placeholder : ''\"\n/>\n\n<input *ngIf=\"type === 'date'\"\n    class=\"ag-table-filter-input\"\n    [control]=\"frmControl\"\n    [date-format]=\"dateFormat\"\n    type=\"text\"\n    [placeholder]=\"placeholder ? placeholder : dateFormat.toUpperCase()\"\n/>\n\n<input *ngIf=\"type === 'number'\"\n    class=\"ag-table-filter-input\"\n    [formControl]=\"frmControl\"\n    type=\"text\"\n    OnlyNumber\n    [placeholder]=\"placeholder ? placeholder : ''\"\n/>\n"
    })
], AgTableFilterComponent);

const TRANSLATION$8 = {};

let AgTableSpinnerComponent = class AgTableSpinnerComponent {
    constructor() {
        this.class = true;
        this.dictionary = TRANSLATION$8;
    }
    ngOnInit() {
    }
};
__decorate([
    HostBinding('class.ag-table-spinner')
], AgTableSpinnerComponent.prototype, "class", void 0);
AgTableSpinnerComponent = __decorate([
    Component({
        selector: 'ag-table-spinner',
        template: "<svg width=\"100\" height=\"100\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">\n\t<circle cx=\"50\" cy=\"50\" r=\"25\" stroke-linecap=\"round\"/>\n</svg>"
    })
], AgTableSpinnerComponent);

let TranslatePipe = class TranslatePipe {
    constructor(langService) {
        this.langService = langService;
    }
    transform(prop, dict = null, replaceObj) {
        let text = '';
        this.langService.getText(prop, dict, false).subscribe(result => {
            if (result !== undefined && typeof (result) === 'string') {
                if (replaceObj && (typeof replaceObj).includes('object')) {
                    for (let property in replaceObj)
                        result = result.replace(`{{${property}}}`, replaceObj[property]);
                }
                text = result;
            }
        });
        return text;
    }
};
TranslatePipe.ctorParameters = () => [
    { type: AgTableLangService }
];
TranslatePipe.ɵprov = ɵɵdefineInjectable({ factory: function TranslatePipe_Factory() { return new TranslatePipe(ɵɵinject(AgTableLangService)); }, token: TranslatePipe, providedIn: "root" });
TranslatePipe = __decorate([
    Injectable({
        providedIn: 'root'
    }),
    Pipe({
        name: 'translate',
        pure: false
    })
], TranslatePipe);

let DateFormatDirective = class DateFormatDirective {
    constructor(helper, settings) {
        this.helper = helper;
        this.settings = settings;
        this.keyCodesAccepts = [8, 9, 37, 38, 39, 40];
        this.ctrlDown = false;
        this.pasting = false;
        this.datePipe = new DatePipe(settings.lang);
    }
    ngOnChanges(changes) {
        if ('dateFormat' in changes) {
            let regexStr = '';
            if (this.dateFormat) {
                if (!this.dateFormat.includes('-') && !this.dateFormat.includes('/')) {
                    if (this.dateFormat.toLowerCase() === 'dd')
                        regexStr += '([0-2]\\d{1}|3[0-1])';
                    else if (this.dateFormat.toLowerCase() === 'mm')
                        regexStr += '(0\\d{1}|1[0-2])';
                    else if (this.dateFormat.toLowerCase() === 'yyyy')
                        regexStr += '\\d{4}';
                }
                else if (this.dateFormat.includes('-') || this.dateFormat.includes('/')) {
                    this.dateSeparator = this.dateFormat.includes('-') ? '-' : '/';
                    let splitted = this.dateFormat.split(this.dateSeparator);
                    splitted.forEach((division) => {
                        let divisionRegex = '';
                        if (division.toLowerCase() === 'dd')
                            divisionRegex = '([0-2]\\d{1}|3[0-1])';
                        else if (division.toLowerCase() === 'mm')
                            divisionRegex = '(0\\d{1}|1[0-2])';
                        else if (division.toLowerCase() === 'yyyy')
                            divisionRegex = '\\d{4}';
                        if (divisionRegex)
                            regexStr += (regexStr !== '' ? this.dateSeparator : '') + divisionRegex;
                    });
                }
            }
            if (regexStr === '') {
                this.dateSeparator = '/';
                this.dateFormat = 'dd/MM/yyyy';
                regexStr = '([0-2]\\d{1}|3[0-1])/(0\d{1}|1[0-2])/\\d{4}';
            }
            this.regex = new RegExp(regexStr, 'g');
        }
    }
    onKeydown(e) {
        let input = e.target;
        let code = e.keyCode;
        let key = e.key.toLowerCase();
        if (code == 17 || code == 91)
            this.ctrlDown = true;
        if (this.keyCodesAccepts.some(x => x === code) || (this.ctrlDown && (code == 86 || code == 67 || key === 'a' || key === 'x'))) {
            if ((this.ctrlDown && key === 'x') || code === 8) {
                if ((input.selectionStart === 0 && input.selectionEnd === input.value.length) || input.value.length === 1)
                    setTimeout(() => this.onBlur(e));
            }
        }
        else {
            let object = this.generateValue(input.value, e.key);
            if (!isNullOrUndefined(object.validValue.match(this.regex))) {
                input.value = object.formatedValue;
                if (input.value.length === this.dateFormat.length)
                    this.onBlur(e);
            }
            event.preventDefault();
            return false;
        }
    }
    onKeyup(e) {
        if (e.keyCode == 17 || e.keyCode == 91)
            this.ctrlDown = false;
    }
    onPaste(e) {
        if (!this.pasting) {
            let input = e.target;
            let prevValue = input.value;
            this.pasting = true;
            setTimeout(() => {
                this.pasting = false;
                let value = this.helper.onlyNumbers(input.value, [this.dateFormat]);
                let object = this.generateValue(value);
                if (!isNullOrUndefined(object.validValue.match(this.regex))) {
                    input.value = object.formatedValue;
                    if (input.value.length === this.dateFormat.length)
                        this.onBlur(e);
                }
                else
                    input.value = prevValue;
            });
        }
    }
    onCut(e) {
        if (!this.pasting) {
            let input = e.target;
            this.pasting = true;
            setTimeout(() => {
                this.pasting = false;
                let value = this.helper.onlyNumbers(input.value, [this.dateFormat]);
                let object = this.generateValue(value);
                if (!isNullOrUndefined(object.validValue.match(this.regex))) {
                    input.value = object.formatedValue;
                }
                else
                    input.value = '';
                this.onBlur(e);
            });
        }
    }
    onBlur(e) {
        if (this.formControl) {
            let input = e.target;
            this.formControl.setValue(input.value);
        }
    }
    generateValue(currentValue, nextChar = null) {
        let object = {
            validValue: currentValue,
            formatedValue: '',
        };
        if (currentValue.length !== this.dateFormat.length) {
            let newValue = '';
            if (this.dateSeparator) {
                for (var i = 0; i < this.dateFormat.length; i++) {
                    let format = this.dateFormat[i];
                    let length = i + 1;
                    if (length <= currentValue.length)
                        newValue += currentValue[i];
                    else if ((length - 1) === currentValue.length) {
                        if (format === this.dateSeparator) {
                            newValue += format;
                            if (nextChar)
                                i++;
                        }
                        newValue += nextChar ? nextChar : '1';
                        object.formatedValue = newValue;
                    }
                    else
                        newValue += (format !== this.dateSeparator) ? '1' : format;
                }
            }
            else {
            }
            object.validValue = newValue;
        }
        if (!object.formatedValue)
            object.formatedValue = object.validValue;
        return object;
    }
    maskDate(value) {
    }
};
DateFormatDirective.ctorParameters = () => [
    { type: Helper },
    { type: AgTableSettings }
];
__decorate([
    Input('date-format')
], DateFormatDirective.prototype, "dateFormat", void 0);
__decorate([
    Input('control')
], DateFormatDirective.prototype, "formControl", void 0);
__decorate([
    HostListener('keydown', ['$event'])
], DateFormatDirective.prototype, "onKeydown", null);
__decorate([
    HostListener('keyup', ['$event'])
], DateFormatDirective.prototype, "onKeyup", null);
__decorate([
    HostListener('paste', ['$event'])
], DateFormatDirective.prototype, "onPaste", null);
__decorate([
    HostListener('cut', ['$event'])
], DateFormatDirective.prototype, "onCut", null);
__decorate([
    HostListener('blur', ['$event'])
], DateFormatDirective.prototype, "onBlur", null);
DateFormatDirective = __decorate([
    Directive({
        selector: '[date-format]'
    })
], DateFormatDirective);

let AgTableModule = class AgTableModule {
};
AgTableModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            RouterModule,
            FormsModule,
            ReactiveFormsModule,
        ],
        declarations: [
            AgTableComponent,
            AgTableHeaderComponent,
            AgTableBodyComponent,
            AgTableRowComponent,
            AgTableCellComponent,
            AgTableColComponent,
            AgTableFilterComponent,
            AgTablePaginateComponent,
            AgTableSpinnerComponent,
            // Pipes
            TranslatePipe,
            // Directives
            DateFormatDirective
        ],
        exports: [
            AgTableComponent,
            AgTableHeaderComponent,
            AgTableBodyComponent,
            AgTableRowComponent,
            AgTableCellComponent,
            AgTableColComponent
        ],
        bootstrap: [],
        providers: [
            AgTablePrepareService,
            AgTableVirtualScrollService,
            AgTableCustomSettings,
            AgTableSettings,
            AgTableLangService,
            Helper
        ],
        entryComponents: []
    }),
    Injectable()
], AgTableModule);

/**
 * Generated bundle index. Do not edit.
 */

export { AgTableCustomSettings, AgTableDataRenderEvent, AgTableEvent, AgTableFilterMode, AgTableFilterType, AgTableModule, AgTableSettingsAbstract, AgTableVirtualScrollModel, FILTER_TYPES, AgTableComponent as ɵa, AgTableHeaderComponent as ɵb, AgTableColComponent as ɵc, Helper as ɵd, AgTableLangService as ɵe, AgTableSettings as ɵf, AgTableBodyComponent as ɵg, AgTableRowComponent as ɵh, AgTableCellComponent as ɵi, AgTableVirtualScrollService as ɵj, AgTablePaginateComponent as ɵk, AgTablePrepareService as ɵl, AgTableFilterComponent as ɵm, AgTableSpinnerComponent as ɵn, TranslatePipe as ɵo, DateFormatDirective as ɵp };
//# sourceMappingURL=ag-table.js.map
